package it.arnaldo.unibs.tamagolem;

import java.util.*;

public class ElementGraph {
    private final Set<GraphElement> elements;

    public ElementGraph() {
        this.elements = new HashSet<>();
    }
    public void addElement(GraphElement element) {
        elements.add(element);
    }

    public int getValue(Element fromElement, Element toElement) {
        for (GraphElement graphElement : elements) {
            if (graphElement.getElement().equals(fromElement)) {
                return graphElement.getLinkValue(toElement);
            }
        }
        return 0;
    }

    public Set<GraphElement> getElements() {
        return elements;
    }

    public void printGraph() {
        for (GraphElement from : elements) {
            Element fromElement = from.getElement();
            System.out.println("[" + fromElement + "] →");

            for (ElementLink link : from.getLinks()) {
                Element toElement = link.getElement();
                int value = link.getValue();
                System.out.printf("   -> %s : %d\n", toElement, value);
            }

            System.out.println();
        }
    }

    public int getTotalStrength(){
        int totalStrength = 0;

        for (GraphElement element : elements) {
            totalStrength += element.getTotalStrength();
        }

        return totalStrength;
    }

    public List<Element> getElementsNames(){
        List<Element> elements = new ArrayList<>();
        for (GraphElement el : this.elements ){
            elements.add(el.getElement());
        }
        return elements;
    }

    public int getInteractionBetween(Element fromElement, Element toElement) {
        for (GraphElement el : elements) {
            if (el.getElement().equals(fromElement)){
                for (ElementLink link : el.getLinks()) {
                    if (link.getElement().equals(toElement)){
                        return link.getValue();
                    }
                }
            }
        }

        return 0;
    }


}
