package it.arnaldo.unibs.tamagolem;

import it.kibo.fp.lib.Menu;

import java.util.HashMap;
import java.util.List;

public class Game {
    private final Player player1;
    private final Player player2;
    private final ElementGraph worldBalance;
    private boolean restart = false;
    private Modes mode;
    private int numberOfElements;
    private int numberOfStones;
    private int numberOfTamaGolems;
    private int totalNumberOfStones;
    private int numberOfStonesPerElement;
    private int totalPower;
    private int lifePoints;
    private HashMap<Element, Integer> stonesPerElement;

    public Game(Player player1, Player player2, ElementGraph worldBalance) {
        this.player1 = player1;
        this.player2 = player2;
        this.worldBalance = worldBalance;
        this.stonesPerElement = new HashMap<>();
    }

    public Player getPlayer1() {
        return player1;
    }

    public Player getPlayer2() {
        return player2;
    }

    public ElementGraph getWorldBalance() {
        return worldBalance;
    }

    public boolean isRestart() {
        return restart;
    }

    public void setRestart(boolean restart) {
        this.restart = restart;
    }

    private void loop(){
        List<Element> elementsNames = worldBalance.getElementsNames();

        // setto la mappa con le pietre ancora disponibili
        for (Element element : elementsNames) {
            stonesPerElement.put(element, numberOfStonesPerElement);
        }

        // aggiungo i tamagolem
        for (int i = 0; i < numberOfTamaGolems; i++) {
            player1.addTamaGolem(new TamaGolem(lifePoints));
            player2.addTamaGolem(new TamaGolem(lifePoints));
        }


        while(!player1.getTamaGolems().isEmpty() && !player2.getTamaGolems().isEmpty()) {
            // setup del primo tamagolem
            setupGolem(player1.getNextTamaGolem());
            setupGolem(player2.getNextTamaGolem());

            FightEsit esito = fight();

            switch (esito) {
                case GOLEM1 -> System.out.println(player1.getName() + " won the first fight");
                case GOLEM2 -> System.out.println(player2.getName() + " won the first fight");
            }

            // dopo il fight i tamagolem morti vengono rimossi
            checkDeathsTamagolems(player1, player2);
        }

        if (player1.getTamaGolems().isEmpty()) {
            System.out.println(player2.getName() + " won the game");
        } else {
            System.out.println(player1.getName() + " won the first game");
        }




        System.out.println("GameLoop");
    }

    private void setupGolem(TamaGolem golem) {
        List<Element> elementsNames = worldBalance.getElementsNames();
        for (int i = 0; i < numberOfStones; i++) {
            Menu stonesMenu = new Menu("TamaGolem", getMenuOptions(), true, true, false);
            int firstmenuChoise = stonesMenu.choose();
            golem.addElementalStone(elementsNames.get(firstmenuChoise - 1));
            stonesPerElement.put(elementsNames.get(firstmenuChoise - 1), stonesPerElement.get(elementsNames.get(firstmenuChoise - 1)) - 1);
        }

        golem.print();
    }


    private void checkDeathsTamagolems(Player p1, Player p2) {
        p1.getTamaGolems().removeIf(tg -> !tg.isAlive());
        p2.getTamaGolems().removeIf(tg -> !tg.isAlive());
    }


    private void setup() {
        // N
        numberOfElements = worldBalance.getElements().size();

        // P
        numberOfStones = (int) Math.ceil((numberOfElements + 1) / 3.0) + 1;

        // G
        numberOfTamaGolems = (int) Math.ceil(((numberOfElements - 1) * (numberOfElements - 2)) / (2.0 * numberOfStones));

        // S
        totalNumberOfStones = (int) Math.ceil((2.0 * numberOfTamaGolems * numberOfStones));

        // Stones for each element
        numberOfStonesPerElement = totalNumberOfStones / numberOfElements;


        // deve essere uguale alla somma della potenza di tutti gli elementi
        totalPower = worldBalance.getTotalStrength();

        lifePoints = totalPower;

    }

    public void start() {
        setup();
        loop();
        worldBalance.printGraph();
    }

    private boolean checkVictory(){
        return !player1.isAlive() || !player2.isAlive();
    }

    private String[] getMenuOptions() {
        List<Element> elementsNames = worldBalance.getElementsNames();
        String[] menuOptions = new String[numberOfElements];

        for (int i = 0; i < numberOfElements; i++) {
            menuOptions[i] = elementsNames.get(i) + " -> (" + stonesPerElement.get(elementsNames.get(i)) + ")";
        }

        return menuOptions;
    }

    public FightEsit fight() {
        TamaGolem golem1 = player1.getNextTamaGolem();
        TamaGolem golem2 = player2.getNextTamaGolem();

        while (golem1.isAlive() && golem2.isAlive()) {
            Element firstStone = golem1.useElementalStone();
            Element secondStone = golem2.useElementalStone();
            int interaction = worldBalance.getInteractionBetween(firstStone, secondStone);

            System.out.println(player1.getName() + "'s tamagolem throws " + firstStone.name() + " and " + player2.getName() + "'s tamagolem throws " + secondStone.name());

            if (interaction > 0){
                golem2.getDamage(interaction);
                System.out.println(player2.getName() + "'s tamagolem gets " + interaction + " damage");
            } else {
                golem1.getDamage(-1 * interaction);
                System.out.println(player1.getName() + "'s tamagolem gets " + -1 * interaction + " damage");
            }
        }

        if (golem1.isAlive()){
            return FightEsit.GOLEM1;
        }
        return FightEsit.GOLEM2;
    }

}
